.bk .markdown {
    padding-right: 1em;
    padding-left: 1em;
}
.markdown a {
    color: var(--accent-foreground-rest);
    text-decoration: underline;
}
.markdown a:hover {
    color: var(--accent-foreground-hover);
}
.markdown a img {
    text-decoration: none
}
.markdown img {
    border-radius: calc(var(--corner-radius) * 1px);
    max-width: 100%;
    margin: 1em;
}
.bk-root .bk.alert {
    padding-left: 1em;
    padding-right: 1em;
}
